@@ -1,6 +1,7 @@
 package testutils
 
 import (
+	"context"
 	"crypto"
 	cryptorand "crypto/rand"
 	"crypto/tls"
@@ -29,7 +30,6 @@ import (
 	"github.com/sirupsen/logrus"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
-	"golang.org/x/net/context"
 	"google.golang.org/grpc"
 	"google.golang.org/grpc/credentials"
 )
