@@ -1,6 +1,7 @@
 package testutils
 
 import (
+	"context"
 	"io/ioutil"
 	"net"
 	"os"
@@ -9,8 +10,6 @@ import (
 	"testing"
 	"time"
 
-	"golang.org/x/net/context"
-
 	"google.golang.org/grpc"
 
 	etcdraft "github.com/coreos/etcd/raft"
@@ -520,12 +519,13 @@ func ProposeValue(t *testing.T, raftNode *TestNode, time time.Duration, nodeID .
 		},
 	}
 
-	ctx, _ := context.WithTimeout(context.Background(), time)
+	ctx, cancel := context.WithTimeout(context.Background(), time)
 
 	err := raftNode.ProposeValue(ctx, storeActions, func() {
 		err := raftNode.MemoryStore().ApplyStoreActions(storeActions)
 		assert.NoError(t, err, "error applying actions")
 	})
+	cancel()
 	if err != nil {
 		return nil, err
 	}
