@@ -2,6 +2,7 @@ package node
 
 import (
 	"bytes"
+	"context"
 	"crypto/tls"
 	"encoding/json"
 	"io/ioutil"
@@ -14,7 +15,9 @@ import (
 	"sync"
 	"time"
 
-	"github.com/boltdb/bolt"
+	"github.com/docker/swarmkit/ca/keyutils"
+	"github.com/docker/swarmkit/identity"
+
 	"github.com/docker/docker/pkg/plugingetter"
 	metrics "github.com/docker/go-metrics"
 	"github.com/docker/swarmkit/agent"
@@ -25,16 +28,17 @@ import (
 	"github.com/docker/swarmkit/ioutils"
 	"github.com/docker/swarmkit/log"
 	"github.com/docker/swarmkit/manager"
+	"github.com/docker/swarmkit/manager/allocator/cnmallocator"
 	"github.com/docker/swarmkit/manager/encryption"
 	"github.com/docker/swarmkit/remotes"
 	"github.com/docker/swarmkit/xnet"
 	grpc_prometheus "github.com/grpc-ecosystem/go-grpc-prometheus"
 	"github.com/pkg/errors"
 	"github.com/sirupsen/logrus"
-	"golang.org/x/net/context"
+	bolt "go.etcd.io/bbolt"
 	"google.golang.org/grpc"
-	"google.golang.org/grpc/codes"
 	"google.golang.org/grpc/credentials"
+	"google.golang.org/grpc/status"
 )
 
 const (
@@ -52,6 +56,9 @@ var (
 
 	// ErrInvalidUnlockKey is returned when we can't decrypt the TLS certificate
 	ErrInvalidUnlockKey = errors.New("node is locked, and needs a valid unlock key")
+
+	// ErrMandatoryFIPS is returned when the cluster we are joining mandates FIPS, but we are running in non-FIPS mode
+	ErrMandatoryFIPS = errors.New("node is not FIPS-enabled but cluster requires FIPS")
 )
 
 func init() {
@@ -99,6 +106,9 @@ type Config struct {
 	// for connections to the remote API (including the raft service).
 	AdvertiseRemoteAPI string
 
+	// NetworkConfig stores network related config for the cluster
+	NetworkConfig *cnmallocator.NetworkConfig
+
 	// Executor specifies the executor to use for the agent.
 	Executor exec.Executor
 
@@ -123,6 +133,9 @@ type Config struct {
 
 	// PluginGetter provides access to docker's plugin inventory.
 	PluginGetter plugingetter.PluginGetter
+
+	// FIPS is a boolean stating whether the node is FIPS enabled
+	FIPS bool
 }
 
 // Node implements the primary node functionality for a member of a swarm
@@ -243,7 +256,6 @@ func (n *Node) Start(ctx context.Context) error {
 		go n.run(ctx)
 		err = nil // clear error above, only once.
 	})
-
 	return err
 }
 
@@ -260,12 +272,16 @@ func (n *Node) currentRole() api.NodeRole {
 func (n *Node) run(ctx context.Context) (err error) {
 	defer func() {
 		n.err = err
+		// close the n.closed channel to indicate that the Node has completely
+		// terminated
 		close(n.closed)
 	}()
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
 	ctx = log.WithModule(ctx, "node")
 
+	// set up a goroutine to monitor the stop channel, and cancel the run
+	// context when the node is stopped
 	go func(ctx context.Context) {
 		select {
 		case <-ctx.Done():
@@ -274,6 +290,17 @@ func (n *Node) run(ctx context.Context) (err error) {
 		}
 	}(ctx)
 
+	// First thing's first: get the SecurityConfig for this node. This includes
+	// the certificate information, and the root CA.  It also returns a cancel
+	// function. This is needed because the SecurityConfig is a live object,
+	// and provides a watch queue so that caller can observe changes to the
+	// security config. This watch queue has to be closed, which is done by the
+	// secConfigCancel function.
+	//
+	// It's also noteworthy that loading the security config with the node's
+	// loadSecurityConfig method has the side effect of setting the node's ID
+	// and role fields, meaning it isn't until after that point that node knows
+	// its ID
 	paths := ca.NewConfigPaths(filepath.Join(n.config.StateDir, certDirectory))
 	securityConfig, secConfigCancel, err := n.loadSecurityConfig(ctx, paths)
 	if err != nil {
@@ -281,11 +308,23 @@ func (n *Node) run(ctx context.Context) (err error) {
 	}
 	defer secConfigCancel()
 
+	// Now that we have the security config, we can get a TLSRenewer, which is
+	// a live component handling certificate rotation.
 	renewer := ca.NewTLSRenewer(securityConfig, n.connBroker, paths.RootCA)
 
+	// Now that we have the security goop all loaded, we know the Node's ID and
+	// can add that to our logging context.
 	ctx = log.WithLogger(ctx, log.G(ctx).WithField("node.id", n.NodeID()))
 
+	// Next, set up the task database. The task database is used by the agent
+	// to keep a persistent local record of its tasks. Since every manager also
+	// has an agent, every node needs a task database, so we do this regardless
+	// of role.
 	taskDBPath := filepath.Join(n.config.StateDir, "worker", "tasks.db")
+	// Doing os.MkdirAll will create the necessary directory path for the task
+	// database if it doesn't already exist, and if it does already exist, no
+	// error will be returned, so we use this regardless of whether this node
+	// is new or not.
 	if err := os.MkdirAll(filepath.Dir(taskDBPath), 0777); err != nil {
 		return err
 	}
@@ -296,8 +335,18 @@ func (n *Node) run(ctx context.Context) (err error) {
 	}
 	defer db.Close()
 
+	// agentDone is a channel that represents the agent having exited. We start
+	// the agent in a goroutine a few blocks down, and before that goroutine
+	// exits, it closes this channel to signal to the goroutine just below to
+	// terminate.
 	agentDone := make(chan struct{})
 
+	// This goroutine is the node changes loop. The n.notifyNodeChange
+	// channel is passed to the agent. When an new node object gets sent down
+	// to the agent, it gets passed back up to this node object, so that we can
+	// check if a role update or a root certificate rotation is required. This
+	// handles root rotation, but the renewer handles regular certification
+	// rotation.
 	go func() {
 		// lastNodeDesiredRole is the last-seen value of Node.Spec.DesiredRole,
 		// used to make role changes "edge triggered" and avoid renewal loops.
@@ -354,9 +403,14 @@ func (n *Node) run(ctx context.Context) (err error) {
 		}
 	}()
 
+	// Now we're going to launch the main component goroutines, the Agent, the
+	// Manager (maybe) and the certificate updates loop. We shouldn't exit
+	// the node object until all 3 of these components have terminated, so we
+	// create a waitgroup to block termination of the node until then
 	var wg sync.WaitGroup
 	wg.Add(3)
 
+	// These two blocks update some of the metrics settings.
 	nodeInfo.WithValues(
 		securityConfig.ClientTLSCreds.Organization(),
 		securityConfig.ClientTLSCreds.NodeID(),
@@ -368,6 +422,10 @@ func (n *Node) run(ctx context.Context) (err error) {
 		nodeManager.Set(0)
 	}
 
+	// We created the renewer way up when we were creating the SecurityConfig
+	// at the beginning of run, but now we're ready to start receiving
+	// CertificateUpdates, and launch a goroutine to handle this. Updates is a
+	// channel we iterate containing the results of certificate renewals.
 	updates := renewer.Start(ctx)
 	go func() {
 		for certUpdate := range updates {
@@ -375,12 +433,14 @@ func (n *Node) run(ctx context.Context) (err error) {
 				logrus.Warnf("error renewing TLS certificate: %v", certUpdate.Err)
 				continue
 			}
+			// Set the new role, and notify our waiting role changing logic
+			// that the role has changed.
 			n.Lock()
 			n.role = certUpdate.Role
 			n.roleCond.Broadcast()
 			n.Unlock()
 
-			// Export the new role.
+			// Export the new role for metrics
 			if n.currentRole() == api.NodeRoleManager {
 				nodeManager.Set(1)
 			} else {
@@ -391,13 +451,19 @@ func (n *Node) run(ctx context.Context) (err error) {
 		wg.Done()
 	}()
 
+	// and, finally, start the two main components: the manager and the agent
 	role := n.role
 
+	// Channels to signal when these respective components are up and ready to
+	// go.
 	managerReady := make(chan struct{})
 	agentReady := make(chan struct{})
+	// these variables are defined in this scope so that they're closed on by
+	// respective goroutines below.
 	var managerErr error
 	var agentErr error
 	go func() {
+		// superviseManager is a routine that watches our manager role
 		managerErr = n.superviseManager(ctx, securityConfig, paths.RootCA, managerReady, renewer) // store err and loop
 		wg.Done()
 		cancel()
@@ -409,6 +475,11 @@ func (n *Node) run(ctx context.Context) (err error) {
 		close(agentDone)
 	}()
 
+	// This goroutine is what signals that the node has fully started by
+	// closing the n.ready channel. First, it waits for the agent to start.
+	// Then, if this node is a manager, it will wait on either the manager
+	// starting, or the node role changing. This ensures that if the node is
+	// demoted before the manager starts, it doesn't get stuck.
 	go func() {
 		<-agentReady
 		if role == ca.ManagerRole {
@@ -428,6 +499,8 @@ func (n *Node) run(ctx context.Context) (err error) {
 		close(n.ready)
 	}()
 
+	// And, finally, we park and wait for the node to close up. If we get any
+	// error other than context canceled, we return it.
 	wg.Wait()
 	if managerErr != nil && errors.Cause(managerErr) != context.Canceled {
 		return managerErr
@@ -435,6 +508,9 @@ func (n *Node) run(ctx context.Context) (err error) {
 	if agentErr != nil && errors.Cause(agentErr) != context.Canceled {
 		return agentErr
 	}
+	// NOTE(dperny): we return err here, but the last time I can see err being
+	// set is when we open the boltdb way up in this method, so I don't know
+	// what returning err is supposed to do.
 	return err
 }
 
@@ -477,11 +553,25 @@ func (n *Node) Err(ctx context.Context) error {
 	}
 }
 
+// runAgent starts the node's agent. When the agent has started, the provided
+// ready channel is closed. When the agent exits, this will return the error
+// that caused it.
 func (n *Node) runAgent(ctx context.Context, db *bolt.DB, securityConfig *ca.SecurityConfig, ready chan<- struct{}) error {
-	waitCtx, waitCancel := context.WithCancel(ctx)
+	// First, get a channel for knowing when a remote peer has been selected.
+	// The value returned from the remotesCh is ignored, we just need to know
+	// when the peer is selected
 	remotesCh := n.remotes.WaitSelect(ctx)
+	// then, we set up a new context to pass specifically to
+	// ListenControlSocket, and start that method to wait on a connection on
+	// the cluster control API.
+	waitCtx, waitCancel := context.WithCancel(ctx)
 	controlCh := n.ListenControlSocket(waitCtx)
 
+	// The goal here to wait either until we have a remote peer selected, or
+	// connection to the control
+	// socket. These are both ways to connect the
+	// agent to a manager, and we need to wait until one or the other is
+	// available to start the agent
 waitPeer:
 	for {
 		select {
@@ -490,20 +580,28 @@ waitPeer:
 		case <-remotesCh:
 			break waitPeer
 		case conn := <-controlCh:
+			// conn will probably be nil the first time we call this, probably,
+			// but only a non-nil conn represent an actual connection.
 			if conn != nil {
 				break waitPeer
 			}
 		}
 	}
 
+	// We can stop listening for new control socket connections once we're
+	// ready
 	waitCancel()
 
+	// NOTE(dperny): not sure why we need to recheck the context here. I guess
+	// it avoids a race if the context was canceled at the same time that a
+	// connection or peer was available. I think it's just an optimization.
 	select {
 	case <-ctx.Done():
 		return ctx.Err()
 	default:
 	}
 
+	// Now we can go ahead and configure, create, and start the agent.
 	secChangesCh, secChangesCancel := securityConfig.Watch()
 	defer secChangesCancel()
 
@@ -523,9 +621,10 @@ waitPeer:
 			CertIssuerPublicKey: issuer.PublicKey,
 			CertIssuerSubject:   issuer.Subject,
 		},
+		FIPS: n.config.FIPS,
 	}
-	// if a join address has been specified, then if the agent fails to connect due to a TLS error, fail fast - don't
-	// keep re-trying to join
+	// if a join address has been specified, then if the agent fails to connect
+	// due to a TLS error, fail fast - don't keep re-trying to join
 	if n.config.JoinAddr != "" {
 		agentConfig.SessionTracker = &firstSessionErrorTracker{}
 	}
@@ -548,6 +647,7 @@ waitPeer:
 		n.Unlock()
 	}()
 
+	// when the agent indicates that it is ready, we close the ready channel.
 	go func() {
 		<-a.Ready()
 		close(ready)
@@ -660,13 +760,36 @@ func (n *Node) Remotes() []api.Peer {
 	return remotes
 }
 
+// Given a cluster ID, returns whether the cluster ID indicates that the cluster
+// mandates FIPS mode.  These cluster IDs start with "FIPS." as a prefix.
+func isMandatoryFIPSClusterID(securityConfig *ca.SecurityConfig) bool {
+	return strings.HasPrefix(securityConfig.ClientTLSCreds.Organization(), "FIPS.")
+}
+
+// Given a join token, returns whether it indicates that the cluster mandates FIPS
+// mode.
+func isMandatoryFIPSClusterJoinToken(joinToken string) bool {
+	if parsed, err := ca.ParseJoinToken(joinToken); err == nil {
+		return parsed.FIPS
+	}
+	return false
+}
+
+func generateFIPSClusterID() string {
+	return "FIPS." + identity.NewID()
+}
+
 func (n *Node) loadSecurityConfig(ctx context.Context, paths *ca.SecurityConfigPaths) (*ca.SecurityConfig, func() error, error) {
 	var (
 		securityConfig *ca.SecurityConfig
 		cancel         func() error
 	)
 
-	krw := ca.NewKeyReadWriter(paths.Node, n.unlockKey, &manager.RaftDEKData{})
+	krw := ca.NewKeyReadWriter(paths.Node, n.unlockKey, &manager.RaftDEKData{FIPS: n.config.FIPS})
+	// if FIPS is required, we want to make sure our key is stored in PKCS8 format
+	if n.config.FIPS {
+		krw.SetKeyFormatter(keyutils.FIPS)
+	}
 	if err := krw.Migrate(); err != nil {
 		return nil, nil, err
 	}
@@ -696,7 +819,7 @@ func (n *Node) loadSecurityConfig(ctx context.Context, paths *ca.SecurityConfigP
 			if n.config.AutoLockManagers {
 				n.unlockKey = encryption.GenerateSecretKey()
 			}
-			krw = ca.NewKeyReadWriter(paths.Node, n.unlockKey, &manager.RaftDEKData{})
+			krw = ca.NewKeyReadWriter(paths.Node, n.unlockKey, &manager.RaftDEKData{FIPS: n.config.FIPS})
 			rootCA, err = ca.CreateRootCA(ca.DefaultRootCN)
 			if err != nil {
 				return nil, nil, err
@@ -706,6 +829,10 @@ func (n *Node) loadSecurityConfig(ctx context.Context, paths *ca.SecurityConfigP
 			}
 			log.G(ctx).Debug("generated CA key and certificate")
 		} else if err == ca.ErrNoLocalRootCA { // from previous error loading the root CA from disk
+			// if we are attempting to join another cluster, which has a FIPS join token, and we are not FIPS, error
+			if n.config.JoinAddr != "" && isMandatoryFIPSClusterJoinToken(n.config.JoinToken) && !n.config.FIPS {
+				return nil, nil, ErrMandatoryFIPS
+			}
 			rootCA, err = ca.DownloadRootCA(ctx, paths.RootCA, n.config.JoinToken, n.connBroker)
 			if err != nil {
 				return nil, nil, err
@@ -730,11 +857,21 @@ func (n *Node) loadSecurityConfig(ctx context.Context, paths *ca.SecurityConfigP
 			}
 			log.G(ctx).WithError(err).Debugf("no node credentials found in: %s", krw.Target())
 
-			securityConfig, cancel, err = rootCA.CreateSecurityConfig(ctx, krw, ca.CertificateRequestConfig{
+			// if we are attempting to join another cluster, which has a FIPS join token, and we are not FIPS, error
+			if n.config.JoinAddr != "" && isMandatoryFIPSClusterJoinToken(n.config.JoinToken) && !n.config.FIPS {
+				return nil, nil, ErrMandatoryFIPS
+			}
+
+			requestConfig := ca.CertificateRequestConfig{
 				Token:        n.config.JoinToken,
 				Availability: n.config.Availability,
 				ConnBroker:   n.connBroker,
-			})
+			}
+			// If this is a new cluster, we want to name the cluster ID "FIPS-something"
+			if n.config.FIPS {
+				requestConfig.Organization = generateFIPSClusterID()
+			}
+			securityConfig, cancel, err = rootCA.CreateSecurityConfig(ctx, krw, requestConfig)
 
 			if err != nil {
 				return nil, nil, err
@@ -742,6 +879,10 @@ func (n *Node) loadSecurityConfig(ctx context.Context, paths *ca.SecurityConfigP
 		}
 	}
 
+	if isMandatoryFIPSClusterID(securityConfig) && !n.config.FIPS {
+		return nil, nil, ErrMandatoryFIPS
+	}
+
 	n.Lock()
 	n.role = securityConfig.ClientTLSCreds.Role()
 	n.nodeID = securityConfig.ClientTLSCreds.NodeID()
@@ -785,6 +926,10 @@ func (n *Node) initManagerConnection(ctx context.Context, ready chan<- struct{})
 	return nil
 }
 
+// waitRole takes a context and a role. it the blocks until the context is
+// canceled or the node's role updates to the provided role. returns nil when
+// the node has acquired the provided role, or ctx.Err() if the context is
+// canceled
 func (n *Node) waitRole(ctx context.Context, role string) error {
 	n.roleCond.L.Lock()
 	if role == n.role {
@@ -814,7 +959,13 @@ func (n *Node) waitRole(ctx context.Context, role string) error {
 	return nil
 }
 
+// runManager runs the manager on this node. It returns a boolean indicating if
+// the stoppage was due to a role change, and an error indicating why the
+// manager stopped
 func (n *Node) runManager(ctx context.Context, securityConfig *ca.SecurityConfig, rootPaths ca.CertPaths, ready chan struct{}, workerRole <-chan struct{}) (bool, error) {
+	// First, set up this manager's advertise and listen addresses, if
+	// provided. they might not be provided if this node is joining the cluster
+	// instead of creating a new one.
 	var remoteAPI *manager.RemoteAddrs
 	if n.config.ListenRemoteAPI != "" {
 		remoteAPI = &manager.RemoteAddrs{
@@ -847,12 +998,21 @@ func (n *Node) runManager(ctx context.Context, securityConfig *ca.SecurityConfig
 		Availability:     n.config.Availability,
 		PluginGetter:     n.config.PluginGetter,
 		RootCAPaths:      rootPaths,
+		FIPS:             n.config.FIPS,
+		NetworkConfig:    n.config.NetworkConfig,
 	})
 	if err != nil {
 		return false, err
 	}
+	// The done channel is used to signal that the manager has exited.
 	done := make(chan struct{})
+	// runErr is an error value set by the goroutine that runs the manager
 	var runErr error
+
+	// The context used to start this might have a logger associated with it
+	// that we'd like to reuse, but we don't want to use that context, so we
+	// pass to the goroutine only the logger, and create a new context with
+	//that logger.
 	go func(logger *logrus.Entry) {
 		if err := m.Run(log.WithLogger(context.Background(), logger)); err != nil {
 			runErr = err
@@ -860,6 +1020,9 @@ func (n *Node) runManager(ctx context.Context, securityConfig *ca.SecurityConfig
 		close(done)
 	}(log.G(ctx))
 
+	// clearData is set in the select below, and is used to signal why the
+	// manager is stopping, and indicate whether or not to delete raft data and
+	// keys when stopping the manager.
 	var clearData bool
 	defer func() {
 		n.Lock()
@@ -877,9 +1040,26 @@ func (n *Node) runManager(ctx context.Context, securityConfig *ca.SecurityConfig
 	connCtx, connCancel := context.WithCancel(ctx)
 	defer connCancel()
 
+	// launch a goroutine that will manage our local connection to the manager
+	// from the agent. Remember the managerReady channel created way back in
+	// run? This is actually where we close it. Not when the manager starts,
+	// but when a connection to the control socket has been established.
 	go n.initManagerConnection(connCtx, ready)
 
 	// wait for manager stop or for role change
+	// The manager can be stopped one of 4 ways:
+	// 1. The manager may have errored out and returned an error, closing the
+	//    done channel in the process
+	// 2. The node may have been demoted to a worker. In this case, we're gonna
+	//    have to stop the manager ourselves, setting clearData to true so the
+	//    local raft data, certs, keys, etc, are nuked.
+	// 3. The manager may have been booted from raft. This could happen if it's
+	//    removed from the raft quorum but the role update hasn't registered
+	//    yet. The fact that there is more than 1 code path to cause the
+	//    manager to exit is a possible source of bugs.
+	// 4. The context may have been canceled from above, in which case we
+	//    should stop the manager ourselves, but indicate that this is NOT a
+	//    demotion.
 	select {
 	case <-done:
 		return false, runErr
@@ -895,12 +1075,22 @@ func (n *Node) runManager(ctx context.Context, securityConfig *ca.SecurityConfig
 	return clearData, nil
 }
 
+// superviseManager controls whether or not we are running a manager on this
+// node
 func (n *Node) superviseManager(ctx context.Context, securityConfig *ca.SecurityConfig, rootPaths ca.CertPaths, ready chan struct{}, renewer *ca.TLSRenewer) error {
+	// superviseManager is a loop, because we can come in and out of being a
+	// manager, and need to appropriately handle that without disrupting the
+	// node functionality.
 	for {
+		// if we're not a manager, we're just gonna park here and wait until we
+		// are. For normal agent nodes, we'll stay here forever, as intended.
 		if err := n.waitRole(ctx, ca.ManagerRole); err != nil {
 			return err
 		}
 
+		// Once we know we are a manager, we get ourselves ready for when we
+		// lose that role. we create a channel to signal that we've become a
+		// worker, and close it when n.waitRole completes.
 		workerRole := make(chan struct{})
 		waitRoleCtx, waitRoleCancel := context.WithCancel(ctx)
 		go func() {
@@ -909,6 +1099,9 @@ func (n *Node) superviseManager(ctx context.Context, securityConfig *ca.Security
 			}
 		}()
 
+		// the ready channel passed to superviseManager is in turn passed down
+		// to the runManager function. It's used to signal to the caller that
+		// the manager has started.
 		wasRemoved, err := n.runManager(ctx, securityConfig, rootPaths, ready, workerRole)
 		if err != nil {
 			waitRoleCancel()
@@ -948,6 +1141,11 @@ func (n *Node) superviseManager(ctx context.Context, securityConfig *ca.Security
 			// re-promoted. In this case, we must assume we were
 			// re-promoted, and restart the manager.
 			log.G(ctx).Warn("failed to get worker role after manager stop, forcing certificate renewal")
+
+			// We can safely reset this timer without stopping/draining the timer
+			// first because the only way the code has reached this point is if the timer
+			// has already expired - if the role changed or the context were canceled,
+			// then we would have returned already.
 			timer.Reset(roleChangeTimeout)
 
 			renewer.Renew()
@@ -967,6 +1165,8 @@ func (n *Node) superviseManager(ctx context.Context, securityConfig *ca.Security
 			return err
 		}
 
+		// set ready to nil after the first time we've gone through this, as we
+		// don't need to signal after the first time that the manager is ready.
 		ready = nil
 	}
 }
@@ -1004,19 +1204,16 @@ func (s *persistentRemotes) Observe(peer api.Peer, weight int) {
 	s.c.Broadcast()
 	if err := s.save(); err != nil {
 		logrus.Errorf("error writing cluster state file: %v", err)
-		return
 	}
-	return
 }
+
 func (s *persistentRemotes) Remove(peers ...api.Peer) {
 	s.Lock()
 	defer s.Unlock()
 	s.Remotes.Remove(peers...)
 	if err := s.save(); err != nil {
 		logrus.Errorf("error writing cluster state file: %v", err)
-		return
 	}
-	return
 }
 
 func (s *persistentRemotes) save() error {
@@ -1100,13 +1297,53 @@ func (fs *firstSessionErrorTracker) SessionError(err error) {
 	fs.mu.Unlock()
 }
 
+// SessionClosed returns an error if we haven't yet established a session, and
+// we get a gprc error as a result of an X509 failure.
 func (fs *firstSessionErrorTracker) SessionClosed() error {
 	fs.mu.Lock()
 	defer fs.mu.Unlock()
-	// unfortunately grpc connection errors are type grpc.rpcError, which are not exposed, and we can't get at the underlying error type
-	if !fs.pastFirstSession && grpc.Code(fs.err) == codes.Internal &&
-		strings.HasPrefix(grpc.ErrorDesc(fs.err), "connection error") && strings.Contains(grpc.ErrorDesc(fs.err), "transport: x509:") {
-		return fs.err
+
+	// if we've successfully established at least 1 session, never return
+	// errors
+	if fs.pastFirstSession {
+		return nil
 	}
-	return nil
+
+	// get the GRPC status from the error, because we only care about GRPC
+	// errors
+	grpcStatus, ok := status.FromError(fs.err)
+	// if this isn't a GRPC error, it's not an error we return from this method
+	if !ok {
+		return nil
+	}
+
+	// NOTE(dperny, cyli): grpc does not expose the error type, which means we have
+	// to string matching to figure out if it's an x509 error.
+	//
+	// The error we're looking for has "connection error:", then says
+	// "transport:" and finally has "x509:"
+	// specifically, the connection error description reads:
+	//
+	//   transport: authentication handshake failed: x509: certificate signed by unknown authority
+	//
+	// This string matching has caused trouble in the past. specifically, at
+	// some point between grpc versions 1.3.0 and 1.7.5, the string we were
+	// matching changed from "transport: x509" to "transport: authentication
+	// handshake failed: x509", which was an issue because we were matching for
+	// string "transport: x509:".
+	//
+	// In GRPC >= 1.10.x, transient errors like TLS errors became hidden by the
+	// load balancing that GRPC does.  In GRPC 1.11.x, they were exposed again
+	// (usually) in RPC calls, but the error string then became:
+	// rpc error: code = Unavailable desc = all SubConns are in TransientFailure, latest connection error: connection error: desc = "transport: authentication handshake failed: x509: certificate signed by unknown authority"
+	//
+	// It also went from an Internal error to an Unavailable error.  So we're just going
+	// to search for the string: "transport: authentication handshake failed: x509:" since
+	// we want to fail for ALL x509 failures, not just unknown authority errors.
+
+	if !strings.Contains(grpcStatus.Message(), "connection error") ||
+		!strings.Contains(grpcStatus.Message(), "transport: authentication handshake failed: x509:") {
+		return nil
+	}
+	return fs.err
 }
